/* 
 * Copyright Marc Guillemot
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.cyberneko.html.xercesbridge;

import org.apache.xerces.impl.Version;
import org.apache.xerces.xni.Augmentations;
import org.apache.xerces.xni.NamespaceContext;
import org.apache.xerces.xni.XMLDocumentHandler;
import org.apache.xerces.xni.XMLLocator;
import org.apache.xerces.xni.parser.XMLDocumentFilter;
import org.apache.xerces.xni.parser.XMLDocumentSource;

/**
 * Xerces bridge for use with Xerces 2.1.<br/>
 * This file won't compile with recent versions of Xerces, this is normal.
 * @author Marc Guillemot
 */
public class XercesBridge_2_1 extends XercesBridge
{
	/**
	 * Should fail for Xerces version less than 2.1 
	 * @throws InstantiationException if instantiation failed 
	 */
	public XercesBridge_2_1() throws InstantiationException {
        try {
            // Just try and see if if we're called with Xerces 2.1 or higher
            getVersion();
        } catch (final Error e) {
            throw new InstantiationException(e.getMessage());
        }
	}

	public String getVersion() {
		return new Version().getVersion();
	}

	public void XMLDocumentHandler_startDocument(XMLDocumentHandler documentHandler, XMLLocator locator,
			String encoding, NamespaceContext nscontext, Augmentations augs) {

		documentHandler.startDocument(locator, encoding, augs);
     }

	public void XMLDocumentFilter_setDocumentSource(XMLDocumentFilter filter,
			XMLDocumentSource lastSource) {
		filter.setDocumentSource(lastSource);
	}
}
